function [fea] = mrmr_mid_d(d, f, K,bdisp)
// MID scheme of  minimum redundancy maximal relevance (mRMR) feature selection
// Calling Sequence
// function [fea] = mrmr_mid_d(d, f, K,verbose)
// Parameters
//  d : a N*M matrix, indicating N samples, each having M dimensions. Must be integers.
//  f : a N*1 matrix (vector), indicating the class/category of the N samples. Must be categorical.
//  K : the number of features need to be selected
//  Description
// The MID scheme of minimum redundancy maximal relevance (mRMR) feature selection
// 
//
//      This version only supports discretized data, thus if you have continuous data in "d", you 
//       will need to discretize them first. 
//
// Bibiliography
//
// H. Peng, F. Long, and C. Ding,  "Feature selection based on mutual information: criteria   of max-dependency, max-relevance, and min-redundancy,"  IEEE Transactions on Pattern Analysis and Machine Intelligence,  Vol. 27, No. 8, pp.1226-1238, 2005. 
//
// C. Ding, and H. Peng,  "Minimum redundancy feature selection from microarray gene   expression data,"   Journal of Bioinformatics and Computational Biology,  Vol. 3, No. 2, pp.185-205, 2005. 
//
// C. Ding, and H. Peng,   "Minimum redundancy feature selection from microarray gene     expression data,"   Proc. 2nd IEEE Computational Systems Bioinformatics Conference (CSB 2003),  pp.523-528, Stanford, CA, Aug, 2003.
//  
// Authors
// Hanchuan Peng (hanchuan.peng@gmail.com) April 16, 2003
//
if argn(2)<4 then
bdisp=0;
end;

nd = size(d,2);
nc = size(d,1);
t=zeros(nd,1);
fea=[];

tic();

for i=1:nd, 
   t(i) = mutualinfo(d(:,i), f);
end; 
printf('calculate the marginal dmi costs %5.1fs.\n', toc());

[tmp, idxs] = mtlb_sort(-t);
fea_base = idxs(1:K);

fea(1) = idxs(1);

KMAX = min(1000,nd); //500

idxleft = idxs(2:KMAX);

k=1;
if bdisp==1,
printf('k=1 cost_time=(N/A) cur_fea=%d #left_cand=%d\n', fea(k), length(idxleft));
end;

for k=2:K,
   tic();
   ncand = length(idxleft);
   curlastfea = length(fea);
   for i=1:ncand,
      t_mi(i) = mutualinfo(d(:,idxleft(i)), f); 
      mi_array(idxleft(i),curlastfea) = getmultimi(d(:,fea(curlastfea)), d(:,idxleft(i)));
      c_mi(i) = mean(mi_array(idxleft(i), :)); 
   end;

   [tmp, fea(k)] = max(t_mi(1:ncand) - c_mi(1:ncand));

   tmpidx = fea(k); fea(k) = idxleft(tmpidx); idxleft(tmpidx) = [];

   if bdisp==1,
   printf('k=%d cost_time=%5.4f cur_fea=%d #left_cand=%d\n',    k, toc(), fea(k), length(idxleft));
   end;
end;

endfunction

//===================================== 
function c = getmultimi(da, dt) 
for i=1:size(da,2), 
   c(i) = mutualinfo(da(:,i), dt);
end; 
endfunction
